@@ -23,10 +23,9 @@ from plone.resource.utils import queryResourceDirectory
 from plone.subrequest import subrequest
 from Products.CMFCore.interfaces import IContentish
 from Products.CMFCore.interfaces import ISiteRoot
+from Products.CMFPlone.utils import safe_encode
 from Products.CMFPlone.utils import safe_unicode
 from Products.PageTemplates.Expressions import getEngine
-from six import StringIO
-from six.moves.configparser import SafeConfigParser
 from six.moves.urllib.parse import urlsplit
 from zope.component import getUtility
 from zope.component import queryMultiAdapter
@@ -37,6 +36,17 @@ import logging
 import pkg_resources
 import six
 
+try:
+    # Python 3.  Watch out for DeprecationWarning:
+    # The SafeConfigParser class has been renamed to ConfigParser in
+    # Python 3.2. This alias will be removed in future versions.
+    # Use ConfigParser directly instead.
+    from configparser import ConfigParser as SafeConfigParser
+except ImportError:
+    # Python 2
+    from ConfigParser import SafeConfigParser
+
+
 LOGGER = logging.getLogger('plone.app.theming')
 
 
@@ -138,7 +148,12 @@ class InternalResolver(etree.Resolver):
         else:
             # e.g. charset=utf-8
             encoding = encoding.split('=', 1)[1].strip()
-        result = result.decode(encoding).encode('ascii', 'xmlcharrefreplace')
+        result = result.decode(encoding)
+        if six.PY2 or content_type == 'text/html':
+            # Note: at first the xmlcharrefreplace was only done on Python 2,
+            # but Python 3 needs it as well, but only for html.
+            # See https://github.com/plone/Products.CMFPlone/issues/3068
+            result = result.encode('ascii', 'xmlcharrefreplace')
 
         if content_type in ('text/javascript', 'application/x-javascript'):
             result = ''.join([
@@ -185,6 +200,27 @@ def findContext(request):
     return request.PARENTS[0]
 
 
+def findPathContext(path):
+    """Find context given by physical path
+    """
+    portal = getPortal()
+
+    if path in (None, '', '/'):
+        return portal
+
+    seq = path.strip('/').split('/')
+    while seq:
+        try:
+            obj = portal.restrictedTraverse('/'.join(seq))
+        except:
+            seq.pop()
+        else:
+            if IContentish.providedBy(obj):
+                return obj
+            else:
+                seq.pop()
+
+
 def expandAbsolutePrefix(prefix):
     """Prepend the Plone site URL to the prefix if it starts with /
     """
@@ -316,9 +352,9 @@ def getTheme(name, manifest=None, resources=None):
     production_js = manifest.get('production-js', None) or ''
     tinymce_content_css = manifest.get('tinymce-content-css', None) or ''
     tinymce_styles_css = manifest.get('tinymce-styles-css', None) or ''
-    if isinstance(rules, str):
+    if isinstance(rules, six.binary_type):
         rules = rules.decode('utf-8')
-    if isinstance(prefix, str):
+    if isinstance(prefix, six.binary_type):
         prefix = prefix.decode('utf-8')
     return Theme(
         name,
@@ -450,13 +486,13 @@ def applyTheme(theme):
 
     else:
 
-        if isinstance(theme.rules, str):
+        if not isinstance(theme.rules, six.text_type):
             theme.rules = theme.rules.decode('utf-8')
 
-        if isinstance(theme.absolutePrefix, str):
+        if not isinstance(theme.absolutePrefix, six.text_type):
             theme.absolutePrefix = theme.absolutePrefix.decode('utf-8')
 
-        if isinstance(theme.__name__, str):
+        if not isinstance(theme.__name__, six.text_type):
             theme.__name__ = theme.__name__.decode('utf-8')
 
         settings.currentTheme = theme.__name__
@@ -494,7 +530,7 @@ def createThemeFromTemplate(title, description, baseOn='template'):
         raise KeyError("Theme {0:s} not found".format(baseOn))
 
     themeName = getUtility(IURLNormalizer).normalize(title)
-    if isinstance(themeName, six.text_type):
+    if six.PY2 and isinstance(themeName, six.text_type):
         themeName = themeName.encode('utf-8')
 
     resources = getOrCreatePersistentResourceDirectory()
@@ -507,18 +543,29 @@ def createThemeFromTemplate(title, description, baseOn='template'):
     manifest = SafeConfigParser()
 
     if MANIFEST_FILENAME in target:
-        fp = target.openFile(MANIFEST_FILENAME)
-        try:
-            manifest.readfp(fp)
-        finally:
-            fp.close()
+        if six.PY2:
+            fp = target.openFile(MANIFEST_FILENAME)
+            try:
+                manifest.readfp(fp)
+            finally:
+                fp.close()
+
+        else:
+            # configparser can only read/write text
+            # but in py3 plone.resource objects are BytesIO objects.
+            fp = target.openFile(MANIFEST_FILENAME)
+            try:
+                data = fp.read()
+            finally:
+                fp.close()
+            manifest.read_string(safe_unicode(data))
 
     if not manifest.has_section('theme'):
         manifest.add_section('theme')
 
-    if not isinstance(title, str):
+    if six.PY2 and isinstance(title, six.text_type):
         title = title.encode('utf-8')
-    if not isinstance(description, str):
+    if six.PY2 and isinstance(description, six.text_type):
         description = description.encode('utf-8')
     manifest.set('theme', 'title', title)
     manifest.set('theme', 'description', description)
@@ -548,10 +595,17 @@ def createThemeFromTemplate(title, description, baseOn='template'):
             val = val.replace(template_prefix, '++%s++%s/' % (THEME_RESOURCE_NAME, themeName))
             manifest.set('theme', var_path, val)
 
-    manifestContents = StringIO()
-    manifest.write(manifestContents)
-    target.writeFile(MANIFEST_FILENAME, manifestContents)
+    # plone.resource uses OFS.File which is a BytesIO objects
+    # but configparser can only deal with text (StringIO).
+    # So we need to do this stupid dance to write manifest.cfg
+    tempfile = six.StringIO()
+    manifest.write(tempfile)
+    tempfile.seek(0)
+    data = tempfile.read()
+    tempfile.close()
+    manifestContents = six.BytesIO(safe_encode(data))
 
+    target.writeFile(MANIFEST_FILENAME, manifestContents)
     return themeName
 
 
